import {AfterViewInit, Component, Inject, LOCALE_ID, OnInit, ViewChild} from '@angular/core';
import {DateHelper, EventStore, LocaleManager, Scheduler} from '@bryntum/scheduler/scheduler.lite.umd.js';
import * as  moment from 'moment';
import {BryntumSchedulerComponent} from '@bryntum/scheduler-angular';
import {MyDrag} from './my-drag';
import schedulerConfig from './scheduler.config';

@Component({
  selector: 'app-scheduler',
  templateUrl: './scheduler.component.html',
  styleUrls: ['./scheduler.component.css']
})
export class SchedulerComponent implements OnInit, AfterViewInit {

  autoRescheduleTasks = false;

  schedulerConfig: any = schedulerConfig;


  @ViewChild(BryntumSchedulerComponent) schedulerComponent: BryntumSchedulerComponent;

  private drag: MyDrag;
  private eventStore: EventStore;
  private scheduler: Scheduler;

  constructor(
    @Inject(LOCALE_ID) locale: string
  ) {
    this.applySchedulerLocale(locale);
  }

  ngOnInit(): void {
    this.setupScheduler();
  }

  ngAfterViewInit(): void {
    this.scheduler = this.schedulerComponent.instance;
    this.eventStore =  this.scheduler.eventStore;

    this.eventStore.on({
      update : ({ record, changes }) => {
        console.log('update');
        if ('resourceId' in changes && !record.resourceId) {
          this.eventStore.remove(record);
          // grid.store.add(record);
        }
        if (this.autoRescheduleTasks) {
          this.rescheduleOverlappingTasks(record);
        }
      },
      add: ({ records }) => {
        console.log('add');
        if (this.autoRescheduleTasks) {
          records.forEach((eventRecord) => this.rescheduleOverlappingTasks(eventRecord));
        }
      }
    });

    this.initMyDrag(this.schedulerComponent?.instance);
  }

  initMyDrag(scheduler): void {
    this.drag = new MyDrag({
      // Don't drag the actual element, clone it
      cloneTarget        : true,
      mode               : 'translateXY',
      // Only allow drops on DOM elements with 'b-timeline-subgrid' CSS class specified
      dropTargetSelector : '.b-timeline-subgrid',
      // Only allow dragging elements with the 'draggable' CSS class
      targetSelector     : '.draggable'
    }, scheduler);
    // drag.setScheduler(scheduler)
  }

  setupScheduler(): void {
    this.schedulerConfig.startDate = moment(new Date()).startOf('month').toDate();
    this.schedulerConfig.endDate = moment(new Date()).endOf('month').toDate();

    this.schedulerConfig.resources = [];
    this.schedulerConfig.events = [];

    this.schedulerConfig.resources.push(
      { id: 1, name: 'Tech 1' },
      { id: 2, name: 'Tech 2' },
      { id: 3, name: 'Tech 3' },
    );
    this.schedulerConfig.events.push({
      id: 1,
      resourceId: 1,
      name: 'Repair fridge',
      startDate: moment(new Date()).startOf('month').toDate(),
      endDate: moment(new Date()).startOf('month').add(10, 'hour').toDate(),
      durationUnit: 'h',
      draggable    : true,
      resizable    : true
    });
  }

  applySchedulerLocale(locale: string): void {
    LocaleManager.registerLocale(locale, { desc : 'Brazil',
      locale : {
        DateHelper : {
          locale
        },
        DependencyEdit : {
          From              : 'De',
          To                : 'Para',
          Type              : 'Tipo',
          Save              : 'Salvar',
          Delete            : 'Deletar',
          Cancel            : 'Cancelar'
        },

        EventEdit : {
          'Edit event' : 'Editar programação',
          Name         : 'Descrição',
          Resource     : 'Técnico',
          Start        : 'Inicio',
          End          : 'Fim',
          Save         : 'Salvar',
          Delete       : 'Deletar',
          Cancel       : 'Cancelar'
        },

        SchedulerBase : {
          'Add event'      : 'Adicionar Programação',
          'Delete event'   : 'Cancelar Programação'
        },

      }
    });

    LocaleManager.locale = locale;
  }


  rescheduleOverlappingTasks(eventRecord): void {
    if (eventRecord.resource) {
      const futureEvents = [];
      const earlierEvents = [];

      // Split tasks into future and earlier tasks
      eventRecord.resource.events.forEach(event => {
        if (event !== eventRecord) {
          if (event.startDate >= eventRecord.startDate) {
            futureEvents.push(event);
          }
          else {
            earlierEvents.push(event);
          }
        }
      });

      if (futureEvents.length || earlierEvents.length) {
        futureEvents.sort((a, b) => a.startDate > b.startDate ? 1 : -1);
        earlierEvents.sort((a, b) => a.startDate > b.startDate ? -1 : 1);

        futureEvents.forEach((ev, i) => {
          const prev = futureEvents[i - 1] || eventRecord;

          ev.startDate = DateHelper.max(prev.endDate, ev.startDate);
        });

        // Walk backwards and remove any overlap
        [eventRecord, ...earlierEvents].forEach((ev, i, all) => {
          const prev = all[i - 1];

          if (ev.endDate > Date.now() && ev !== eventRecord && prev) {
            ev.setEndDate(DateHelper.min(prev.startDate, ev.endDate), true);
          }
        });
      }
    }
  }

}
